@@ -116,7 +116,7 @@ kdebug_sadb(base)
 	    base->sadb_msg_seq, base->sadb_msg_pid);
 
 	tlen = PFKEY_UNUNIT64(base->sadb_msg_len) - sizeof(struct sadb_msg);
-	ext = (struct sadb_ext *)((caddr_t)base + sizeof(struct sadb_msg));
+	ext = (void *)((caddr_t)(void *)base + sizeof(struct sadb_msg));
 
 	while (tlen > 0) {
 		printf("sadb_ext{ len=%u type=%u }\n",
@@ -191,7 +191,7 @@ kdebug_sadb(base)
 
 		extlen = PFKEY_UNUNIT64(ext->sadb_ext_len);
 		tlen -= extlen;
-		ext = (struct sadb_ext *)((caddr_t)ext + extlen);
+		ext = (void *)((caddr_t)(void *)ext + extlen);
 	}
 
 	return;
@@ -201,7 +201,7 @@ static void
 kdebug_sadb_prop(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_prop *prop = (struct sadb_prop *)ext;
+	struct sadb_prop *prop = (void *)ext;
 	struct sadb_comb *comb;
 	int len;
 
@@ -211,7 +211,7 @@ kdebug_sadb_prop(ext)
 
 	len = (PFKEY_UNUNIT64(prop->sadb_prop_len) - sizeof(*prop))
 		/ sizeof(*comb);
-	comb = (struct sadb_comb *)(prop + 1);
+	comb = (void *)(prop + 1);
 	printf("sadb_prop{ replay=%u\n", prop->sadb_prop_replay);
 
 	while (len--) {
@@ -251,7 +251,7 @@ static void
 kdebug_sadb_identity(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_ident *id = (struct sadb_ident *)ext;
+	struct sadb_ident *id = (void *)ext;
 	int len;
 
 	/* sanity check */
@@ -271,7 +271,7 @@ kdebug_sadb_identity(ext)
 #else
 			char *p, *ep;
 			printf("\n  str=\"");
-			p = (char *)(id + 1);
+			p = (void *)(id + 1);
 			ep = p + len;
 			for (/*nothing*/; *p && p < ep; p++) {
 				if (isprint((int)*p))
@@ -294,7 +294,7 @@ static void
 kdebug_sadb_supported(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_supported *sup = (struct sadb_supported *)ext;
+	struct sadb_supported *sup = (void *)ext;
 	struct sadb_alg *alg;
 	int len;
 
@@ -304,7 +304,7 @@ kdebug_sadb_supported(ext)
 
 	len = (PFKEY_UNUNIT64(sup->sadb_supported_len) - sizeof(*sup))
 		/ sizeof(*alg);
-	alg = (struct sadb_alg *)(sup + 1);
+	alg = (void *)(sup + 1);
 	printf("sadb_sup{\n");
 	while (len--) {
 		printf("  { id=%d ivlen=%d min=%d max=%d }\n",
@@ -321,7 +321,7 @@ static void
 kdebug_sadb_lifetime(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_lifetime *lft = (struct sadb_lifetime *)ext;
+	struct sadb_lifetime *lft = (void *)ext;
 
 	/* sanity check */
 	if (ext == NULL)
@@ -341,7 +341,7 @@ static void
 kdebug_sadb_sa(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_sa *sa = (struct sadb_sa *)ext;
+	struct sadb_sa *sa = (void *)ext;
 
 	/* sanity check */
 	if (ext == NULL)
@@ -360,7 +360,7 @@ static void
 kdebug_sadb_address(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_address *addr = (struct sadb_address *)ext;
+	struct sadb_address *addr = (void *)ext;
 
 	/* sanity check */
 	if (ext == NULL)
@@ -368,10 +368,10 @@ kdebug_sadb_address(ext)
 
 	printf("sadb_address{ proto=%u prefixlen=%u reserved=0x%02x%02x }\n",
 	    addr->sadb_address_proto, addr->sadb_address_prefixlen,
-	    ((u_char *)&addr->sadb_address_reserved)[0],
-	    ((u_char *)&addr->sadb_address_reserved)[1]);
+	    ((u_char *)(void *)&addr->sadb_address_reserved)[0],
+	    ((u_char *)(void *)&addr->sadb_address_reserved)[1]);
 
-	kdebug_sockaddr((struct sockaddr *)((caddr_t)ext + sizeof(*addr)));
+	kdebug_sockaddr((void *)((caddr_t)(void *)ext + sizeof(*addr)));
 
 	return;
 }
@@ -380,7 +380,7 @@ static void
 kdebug_sadb_key(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_key *key = (struct sadb_key *)ext;
+	struct sadb_key *key = (void *)ext;
 
 	/* sanity check */
 	if (ext == NULL)
@@ -391,15 +391,15 @@ kdebug_sadb_key(ext)
 	printf("  key=");
 
 	/* sanity check 2 */
-	if ((key->sadb_key_bits >> 3) >
+	if (((uint32_t)key->sadb_key_bits >> 3) >
 		(PFKEY_UNUNIT64(key->sadb_key_len) - sizeof(struct sadb_key))) {
 		printf("kdebug_sadb_key: key length mismatch, bit:%d len:%ld.\n",
-			key->sadb_key_bits >> 3,
+			(uint32_t)key->sadb_key_bits >> 3,
 			(long)PFKEY_UNUNIT64(key->sadb_key_len) - sizeof(struct sadb_key));
 	}
 
-	ipsec_hexdump((caddr_t)key + sizeof(struct sadb_key),
-	              key->sadb_key_bits >> 3);
+	ipsec_hexdump(key + sizeof(struct sadb_key),
+	              (int)((uint32_t)key->sadb_key_bits >> 3));
 	printf(" }\n");
 	return;
 }
@@ -408,7 +408,7 @@ static void
 kdebug_sadb_x_sa2(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_x_sa2 *sa2 = (struct sadb_x_sa2 *)ext;
+	struct sadb_x_sa2 *sa2 = (void *)ext;
 
 	/* sanity check */
 	if (ext == NULL)
@@ -427,7 +427,7 @@ void
 kdebug_sadb_x_policy(ext)
 	struct sadb_ext *ext;
 {
-	struct sadb_x_policy *xpl = (struct sadb_x_policy *)ext;
+	struct sadb_x_policy *xpl = (void *)ext;
 	struct sockaddr *addr;
 
 	/* sanity check */
@@ -451,7 +451,7 @@ kdebug_sadb_x_policy(ext)
 		struct sadb_x_ipsecrequest *xisr;
 
 		tlen = PFKEY_UNUNIT64(xpl->sadb_x_policy_len) - sizeof(*xpl);
-		xisr = (struct sadb_x_ipsecrequest *)(xpl + 1);
+		xisr = (void *)(xpl + 1);
 
 		while (tlen > 0) {
 			printf(" { len=%u proto=%u mode=%u level=%u reqid=%u\n",
@@ -462,9 +462,9 @@ kdebug_sadb_x_policy(ext)
 				xisr->sadb_x_ipsecrequest_reqid);
 
 			if (xisr->sadb_x_ipsecrequest_len > sizeof(*xisr)) {
-				addr = (struct sockaddr *)(xisr + 1);
+				addr = (void *)(xisr + 1);
 				kdebug_sockaddr(addr);
-				addr = (struct sockaddr *)((caddr_t)addr
+				addr = (void *)((caddr_t)(void *)addr
 							+ sysdep_sa_len(addr));
 				kdebug_sockaddr(addr);
 			}
@@ -472,7 +472,7 @@ kdebug_sadb_x_policy(ext)
 			printf(" }\n");
 
 			/* prevent infinite loop */
-			if (xisr->sadb_x_ipsecrequest_len <= 0) {
+			if (xisr->sadb_x_ipsecrequest_len == 0) {
 				printf("kdebug_sadb_x_policy: wrong policy struct.\n");
 				return;
 			}
@@ -484,7 +484,7 @@ kdebug_sadb_x_policy(ext)
 
 			tlen -= xisr->sadb_x_ipsecrequest_len;
 
-			xisr = (struct sadb_x_ipsecrequest *)((caddr_t)xisr
+			xisr = (void *)((caddr_t)(void *)xisr
 			                + xisr->sadb_x_ipsecrequest_len);
 		}
 
@@ -499,7 +499,7 @@ kdebug_sadb_x_policy(ext)
 static void
 kdebug_sadb_x_nat_t_type(struct sadb_ext *ext)
 {
-	struct sadb_x_nat_t_type *ntt = (struct sadb_x_nat_t_type *)ext;
+	struct sadb_x_nat_t_type *ntt = (void *)ext;
 
 	/* sanity check */
 	if (ext == NULL)
@@ -513,7 +513,7 @@ kdebug_sadb_x_nat_t_type(struct sadb_ext *ext)
 static void
 kdebug_sadb_x_nat_t_port(struct sadb_ext *ext)
 {
-	struct sadb_x_nat_t_port *ntp = (struct sadb_x_nat_t_port *)ext;
+	struct sadb_x_nat_t_port *ntp = (void *)ext;
 
 	/* sanity check */
 	if (ext == NULL)
@@ -762,18 +762,17 @@ kdebug_sockaddr(addr)
 
 	switch (addr->sa_family) {
 	case AF_INET:
-		sin4 = (struct sockaddr_in *)addr;
+		sin4 = (void *)addr;
 		printf(" port=%u\n", ntohs(sin4->sin_port));
-		ipsec_hexdump((caddr_t)&sin4->sin_addr, sizeof(sin4->sin_addr));
+		ipsec_hexdump(&sin4->sin_addr, sizeof(sin4->sin_addr));
 		break;
 #ifdef INET6
 	case AF_INET6:
-		sin6 = (struct sockaddr_in6 *)addr;
+		sin6 = (void *)addr;
 		printf(" port=%u\n", ntohs(sin6->sin6_port));
 		printf("  flowinfo=0x%08x, scope_id=0x%08x\n",
 		    sin6->sin6_flowinfo, sin6->sin6_scope_id);
-		ipsec_hexdump((caddr_t)&sin6->sin6_addr,
-		    sizeof(sin6->sin6_addr));
+		ipsec_hexdump(&sin6->sin6_addr, sizeof(sin6->sin6_addr));
 		break;
 #endif
 	}
@@ -799,7 +798,7 @@ ipsec_bindump(buf, len)
 
 void
 ipsec_hexdump(buf, len)
-	caddr_t buf;
+	const void *buf;
 	int len;
 {
 	int i;
@@ -807,7 +806,7 @@ ipsec_hexdump(buf, len)
 	for (i = 0; i < len; i++) {
 		if (i != 0 && i % 32 == 0) printf("\n");
 		if (i % 4 == 0) printf(" ");
-		printf("%02x", (unsigned char)buf[i]);
+		printf("%02x", ((const unsigned char *)buf)[i]);
 	}
 #if 0
 	if (i % 32 != 0) printf("\n");
