@@ -68,7 +68,7 @@
 #define ATOX(c) \
   (isdigit(c) ? (c - '0') : (isupper(c) ? (c - 'A' + 10) : (c - 'a' + 10) ))
 
-static caddr_t pbuf = NULL;		/* sadb_x_policy buffer */
+static u_int8_t *pbuf = NULL;		/* sadb_x_policy buffer */
 static int tlen = 0;			/* total length of pbuf */
 static int offset = 0;			/* offset of pbuf */
 static int p_dir, p_type, p_protocol, p_mode, p_level, p_reqid;
@@ -83,12 +83,12 @@ static struct sockaddr *parse_sockaddr __P((struct _val *addrbuf,
     struct _val *portbuf));
 static int rule_check __P((void));
 static int init_x_policy __P((void));
-static int set_x_request __P((struct sockaddr *src, struct sockaddr *dst));
-static int set_sockaddr __P((struct sockaddr *addr));
+static int set_x_request __P((struct sockaddr *, struct sockaddr *));
+static int set_sockaddr __P((struct sockaddr *));
 static void policy_parse_request_init __P((void));
-static caddr_t policy_parse __P((char *msg, int msglen));
+static void *policy_parse __P((const char *, int));
 
-extern void __policy__strbuffer__init__ __P((char *msg));
+extern void __policy__strbuffer__init__ __P((const char *));
 extern void __policy__strbuffer__free__ __P((void));
 extern int yyparse __P((void));
 extern int yylex __P((void));
@@ -1424,7 +1424,7 @@ set_x_request(src, dst)
 {
 	struct sadb_x_ipsecrequest *p;
 	int reqlen;
-	caddr_t n;
+	u_int8_t *n;
 
 	reqlen = sizeof(*p)
 		+ (src ? sysdep_sa_len(src) : 0)
@@ -1491,9 +1491,9 @@ policy_parse_request_init()
 	return;
 }
 
-static caddr_t
+static void *
 policy_parse(msg, msglen)
-	char *msg;
+	const char *msg;
 	int msglen;
 {
 	int error;
@@ -1524,9 +1524,9 @@ policy_parse(msg, msglen)
 	return pbuf;
 }
 
-caddr_t
+ipsec_policy_t
 ipsec_set_policy(msg, msglen)
-	char *msg;
+	__ipsec_const char *msg;
 	int msglen;
 {
 	caddr_t policy;
@@ -1541,4 +1541,3 @@ ipsec_set_policy(msg, msglen)
 	__ipsec_errcode = EIPSEC_NO_ERROR;
 	return policy;
 }
-
