@@ -1,4 +1,4 @@
-/* $Id: crypto_openssl.c,v 1.40.4.3 2005/04/21 09:07:19 monas Exp $ */
+/* $Id: crypto_openssl.c,v 1.40.4.5 2005/07/12 11:50:15 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -797,7 +797,7 @@ mem2x509(cert)
 
 	bp = (unsigned char *) cert->v;
 
-	x509 = d2i_X509(NULL, &bp, cert->l);
+	x509 = d2i_X509(NULL, (void *)&bp, cert->l);
     }
 #else
     {
@@ -885,7 +885,7 @@ eay_check_x509sign(source, sig, cert)
 
 	bp = (unsigned char *) cert->v;
 
-	x509 = d2i_X509(NULL, &bp, cert->l);
+	x509 = d2i_X509(NULL, (void *)&bp, cert->l);
 	if (x509 == NULL) {
 		plog(LLV_ERROR, LOCATION, NULL, "d2i_X509(): %s\n", eay_strerror());
 		return -1;
@@ -1034,7 +1034,7 @@ eay_get_x509sign(src, privkey)
 	int pad = RSA_PKCS1_PADDING;
 
 	/* XXX to be handled EVP_PKEY_DSA */
-	evp = d2i_PrivateKey(EVP_PKEY_RSA, NULL, &bp, privkey->l);
+	evp = d2i_PrivateKey(EVP_PKEY_RSA, NULL, (void *)&bp, privkey->l);
 	if (evp == NULL)
 		return NULL;
 
@@ -1159,15 +1159,58 @@ evp_crypt(vchar_t *data, vchar_t *key, vchar_t *iv, const EVP_CIPHER *e, int enc
 
 	EVP_CIPHER_CTX_init(&ctx);
 
-	if (!EVP_CipherInit(&ctx, e, (unsigned char *) key->v, 
-				(unsigned char *) iv->v, enc)) {
-		OpenSSL_BUG();
-		vfree(res);
-		return NULL;
+	switch(EVP_CIPHER_nid(e)){
+	case NID_bf_cbc:
+	case NID_bf_ecb:
+	case NID_bf_cfb64:
+	case NID_bf_ofb64:
+	case NID_cast5_cbc:
+	case NID_cast5_ecb:
+	case NID_cast5_cfb64:
+	case NID_cast5_ofb64:
+		/* XXX: can we do that also for algos with a fixed key size ?
+		 */
+		/* init context without key/iv
+         */
+        if (!EVP_CipherInit(&ctx, e, NULL, NULL, enc))
+        {
+            OpenSSL_BUG();
+            vfree(res);
+            return NULL;
+        }
+		
+        /* update key size
+         */
+        if (!EVP_CIPHER_CTX_set_key_length(&ctx, key->l))
+        {
+            OpenSSL_BUG();
+            vfree(res);
+            return NULL;
+        }
+
+        /* finalize context init with desired key size
+         */
+        if (!EVP_CipherInit(&ctx, NULL, (u_char *) key->v,
+							(u_char *) iv->v, enc))
+        {
+            OpenSSL_BUG();
+            vfree(res);
+            return NULL;
+		}
+		break;
+	default:
+		if (!EVP_CipherInit(&ctx, e, (u_char *) key->v, 
+							(u_char *) iv->v, enc)) {
+			OpenSSL_BUG();
+			vfree(res);
+			return NULL;
+		}
 	}
+
+	/* disable openssl padding */
+	EVP_CIPHER_CTX_set_padding(&ctx, 0); 
 	
-	if (!EVP_Cipher(&ctx, (unsigned char *) res->v, 
-				(unsigned char *) data->v, data->l)) {
+	if (!EVP_Cipher(&ctx, (u_char *) res->v, (u_char *) data->v, data->l)) {
 		OpenSSL_BUG();
 		vfree(res);
 		return NULL;
