@@ -1,7 +1,7 @@
-/* $Id: isakmp_cfg.c,v 1.26.2.5 2005/05/10 09:45:46 manubsd Exp $ */
+/* $Id: isakmp_cfg.c,v 1.26.2.7 2006/01/07 23:50:42 manubsd Exp $ */
 
 /*
- * Copyright (C) 2004 Emmanuel Dreyfus
+ * Copyright (C) 2004-2006 Emmanuel Dreyfus
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -728,6 +728,12 @@ isakmp_cfg_net(iph1, attr)
 			/* FALLTHROUGH */
 #endif
 		case ISAKMP_CFG_CONF_LOCAL:
+			if (isakmp_cfg_getport(iph1) == -1) {
+				plog(LLV_ERROR, LOCATION, NULL, 
+				    "Port pool depleted\n");
+				break;
+			}
+
 			iph1->mode_cfg->addr4.s_addr = 
 			    htonl(ntohl(isakmp_cfg_config.network4) 
 			    + iph1->mode_cfg->port);
@@ -1155,6 +1161,9 @@ isakmp_cfg_getport(iph1)
 	unsigned int i;
 	size_t size = isakmp_cfg_config.pool_size;
 
+	if (iph1->mode_cfg->flags & ISAKMP_CFG_PORT_ALLOCATED)
+		return iph1->mode_cfg->port;
+
 	if (isakmp_cfg_config.port_pool == NULL) {
 		plog(LLV_ERROR, LOCATION, NULL,
 		    "isakmp_cfg_config.port_pool == NULL\n");
@@ -1547,10 +1556,21 @@ isakmp_cfg_setenv(iph1, envp, envc)
 	else
 		addrstr[0] = '\0';
 
-	if (script_env_append(envp, envc, "INTERNAL_MASK4", addrstr) != 0) { 
-		plog(LLV_ERROR, LOCATION, NULL, "Cannot set INTERNAL_MASK4\n");
-		return -1;
-	}
+       /* 
+	* During several releases, documentation adverised INTERNAL_NETMASK4
+	* while code was using INTERNAL_MASK4. We now do both.
+	*/
+	if (script_env_append(envp, envc, "INTERNAL_MASK4", addrstr) != 0) {
+                plog(LLV_ERROR, LOCATION, NULL, "Cannot set INTERNAL_MASK4\n");
+                return -1;
+        } 
+
+       if (script_env_append(envp, envc, "INTERNAL_NETMASK4", addrstr) != 0) {
+	       plog(LLV_ERROR, LOCATION, NULL,  
+		   "Cannot set INTERNAL_NETMASK4\n");
+	       return -1;
+       }
+
 
 	/* Internal IPv4 DNS */
 	if (iph1->mode_cfg->flags & ISAKMP_CFG_GOT_DNS4) 
