@@ -1,4 +1,4 @@
-/* $Id: pfkey.c,v 1.31.2.10 2005/10/03 14:52:19 manubsd Exp $ */
+/* $Id: pfkey.c,v 1.31.2.12 2006/05/05 09:09:03 vanhu Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -874,6 +874,16 @@ pk_sendgetspi(iph2)
 			return -1;
 		}
 
+#ifdef ENABLE_NATT
+		/* XXX should we do a copy of src/dst for each pr ?
+		 */
+		if (! pr->udp_encap) {
+			/* Remove port information, that SA doesn't use it */
+			set_port(src, 0);
+			set_port(dst, 0);
+		}
+#endif
+
 		plog(LLV_DEBUG, LOCATION, NULL, "call pfkey_send_getspi\n");
 		if (pfkey_send_getspi(
 				lcconf->sock_pfkey,
@@ -1081,6 +1091,10 @@ pk_sendupdate(iph2)
 			natt.frag = iph2->ph1->rmconf->esp_frag;
 		} else {
 			memset (&natt, 0, sizeof (natt));
+
+			/* Remove port information, that SA doesn't use it */
+			set_port(src, 0);
+			set_port(dst, 0);
 		}
 
 		if (pfkey_send_update_nat(
