@@ -1,4 +1,4 @@
-/* $Id: plog.c,v 1.6 2004/07/12 20:15:08 ludvigm Exp $ */
+/* $Id: plog.c,v 1.6.10.1 2005/12/07 10:19:51 vanhu Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -127,6 +127,10 @@ plog_common(pri, fmt, func)
 		snprintf(p, reslen, "%s: %s", func, fmt);
 	else
 		snprintf(p, reslen, "%s", fmt);
+#ifdef BROKEN_PRINTF
+    while ((p = strstr(buf,"%z")) != NULL)
+		p[1] = 'l';
+#endif
 
 	return buf;
 }
