@@ -59,6 +59,8 @@
 #include <unistd.h>
 #endif
 #include <signal.h>
+#include <sys/stat.h>
+#include <paths.h>
 
 #include "libpfkey.h"
 
@@ -100,7 +102,7 @@ static int close_sockets __P((void));
 static fd_set mask0;
 static fd_set maskdying;
 static int nfds = 0;
-static int sigreq = 0;
+static volatile sig_atomic_t sigreq[NSIG + 1];
 static int dying = 0;
 
 int
@@ -110,6 +112,10 @@ session(void)
 	struct timeval *timeout;
 	int error;
 	struct myaddrs *p;
+	char pid_file[MAXPATHLEN];
+	FILE *fp;
+	pid_t racoon_pid = 0;
+	int i;
 
 	/* initialize schedular */
 	sched_init();
@@ -135,7 +141,34 @@ session(void)
 	if (privsep_init() != 0)
 		exit(1);
 
-	sigreq = 0;
+	for (i = 0; i <= NSIG; i++)
+		sigreq[i] = 0;
+
+	/* write .pid file */
+	racoon_pid = getpid();
+	if (lcconf->pathinfo[LC_PATHTYPE_PIDFILE] == NULL) 
+		strlcpy(pid_file, _PATH_VARRUN "racoon.pid", MAXPATHLEN);
+	else if (lcconf->pathinfo[LC_PATHTYPE_PIDFILE][0] == '/') 
+		strlcpy(pid_file, lcconf->pathinfo[LC_PATHTYPE_PIDFILE], MAXPATHLEN);
+	else {
+		strlcat(pid_file, _PATH_VARRUN, MAXPATHLEN);
+		strlcat(pid_file, lcconf->pathinfo[LC_PATHTYPE_PIDFILE], MAXPATHLEN);
+	} 
+	fp = fopen(pid_file, "w");
+	if (fp) {
+		if (fchmod(fileno(fp),
+			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH) == -1) {
+			syslog(LOG_ERR, "%s", strerror(errno));
+			fclose(fp);
+			exit(1);
+		}
+		fprintf(fp, "%ld\n", (long)racoon_pid);
+		fclose(fp);
+	} else {
+		plog(LLV_ERROR, LOCATION, NULL,
+			"cannot open %s", pid_file);
+	}
+
 	while (1) {
 		if (dying)
 			rfds = maskdying;
@@ -166,7 +199,8 @@ session(void)
 		}
 
 #ifdef ENABLE_ADMINPORT
-		if (FD_ISSET(lcconf->sock_admin, &rfds))
+		if ((lcconf->sock_admin != -1) &&
+		    (FD_ISSET(lcconf->sock_admin, &rfds)))
 			admin_handler();
 #endif
 
@@ -224,17 +258,19 @@ initfds()
 	FD_ZERO(&maskdying);
 
 #ifdef ENABLE_ADMINPORT
-	if (lcconf->sock_admin >= FD_SETSIZE) {
-		plog(LLV_ERROR, LOCATION, NULL, "fd_set overrun\n");
-		exit(1);
-	}
-	FD_SET(lcconf->sock_admin, &mask0);
-	/* XXX should we listen on admin socket when dying ?
-	 */
+	if (lcconf->sock_admin != -1) {
+		if (lcconf->sock_admin >= FD_SETSIZE) {
+			plog(LLV_ERROR, LOCATION, NULL, "fd_set overrun\n");
+			exit(1);
+		}
+		FD_SET(lcconf->sock_admin, &mask0);
+		/* XXX should we listen on admin socket when dying ?
+		 */
 #if 0
-	FD_SET(lcconf->sock_admin, &maskdying);
+		FD_SET(lcconf->sock_admin, &maskdying);
 #endif
-	nfds = (nfds > lcconf->sock_admin ? nfds : lcconf->sock_admin);
+		nfds = (nfds > lcconf->sock_admin ? nfds : lcconf->sock_admin);
+	}
 #endif
 	if (lcconf->sock_pfkey >= FD_SETSIZE) {
 		plog(LLV_ERROR, LOCATION, NULL, "fd_set overrun\n");
@@ -283,57 +319,85 @@ RETSIGTYPE
 signal_handler(sig)
 	int sig;
 {
-	switch (sig) {
-	case SIGCHLD:
-	    {
-		pid_t pid;
-		int s;
-
-		pid = wait(&s);
-	    }
-		break;
-
-#ifdef DEBUG_RECORD_MALLOCATION
-	case SIGUSR2:
-		DRM_dump();
-		break;
-#endif
-	default:
-		/* XXX should be blocked any signal ? */
-		sigreq = sig;
-		break;
-	}
+	/* Do not just set it to 1, because we may miss some signals by just setting
+	 * values to 0/1
+	 */
+	sigreq[sig]++;
 }
 
 static void
 check_sigreq()
 {
-	switch (sigreq) {
-	case 0:
-		return;
+	int sig;
 
-	case SIGHUP:
-		isakmp_close();
-		close(lcconf->rtsock);
-		if (cfreparse()) {
-			plog(LLV_ERROR, LOCATION, NULL,
-				"configuration read failed\n");
-			exit(1);
+	/* 
+	 * XXX We are not able to tell if we got 
+	 * several time the same signal. This is
+	 * not a problem for the current code, 
+	 * but we shall remember this limitation.
+	 */
+	for (sig = 0; sig <= NSIG; sig++) {
+		if (sigreq[sig] == 0)
+			continue;
+
+		sigreq[sig]--;
+		switch(sig) {
+		case 0:
+			return;
+			
+			/* Catch up childs, mainly scripts.
+			 */
+		case SIGCHLD:
+	    {
+			pid_t pid;
+			int s;
+			
+			pid = wait(&s);
+	    }
+		break;
+
+#ifdef DEBUG_RECORD_MALLOCATION
+		/* 
+		 * XXX This operation is signal handler unsafe and may lead to 
+		 * crashes and security breaches: See Henning Brauer talk at
+		 * EuroBSDCon 2005. Do not run in production with this option
+		 * enabled.
+		 */
+		case SIGUSR2:
+			DRM_dump();
+			break;
+#endif
+
+		case SIGHUP:
+			/* Save old configuration, load new one...  */
+			isakmp_close();
+			close(lcconf->rtsock);
+			if (cfreparse()) {
+				plog(LLV_ERROR, LOCATION, NULL,
+					 "configuration read failed\n");
+				exit(1);
+			}
+			initmyaddr();
+			isakmp_init();
+			initfds();
+			break;
+
+		case SIGINT:
+		case SIGTERM:			
+			plog(LLV_INFO, LOCATION, NULL, 
+			    "caught signal %d\n", sig);
+			EVT_PUSH(NULL, NULL, EVTT_RACOON_QUIT, NULL);
+			pfkey_send_flush(lcconf->sock_pfkey, 
+			    SADB_SATYPE_UNSPEC);
+			sched_new(1, check_flushsa_stub, NULL);
+			dying = 1;
+			break;
+
+		default:
+			plog(LLV_INFO, LOCATION, NULL, 
+			    "caught signal %d\n", sig);
+			break;
 		}
-		initmyaddr();
-		isakmp_init();
-		initfds();
-		sigreq = 0;
-		break;
-
-	default:
-		plog(LLV_INFO, LOCATION, NULL, "caught signal %d\n", sigreq);
-		EVT_PUSH(NULL, NULL, EVTT_RACOON_QUIT, NULL);
-		pfkey_send_flush(lcconf->sock_pfkey, SADB_SATYPE_UNSPEC);
-		sched_new(1, check_flushsa_stub, NULL);
-		sigreq = 0;
-		dying = 1;
-		break;
 	}
 }
 
