@@ -1,4 +1,4 @@
-/* $Id: sockmisc.c,v 1.17.4.2 2005/05/14 06:34:29 monas Exp $ */
+/* $Id: sockmisc.c,v 1.17.4.4 2005/10/04 09:54:27 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -312,7 +312,7 @@ recvfromto(s, buf, buflen, flags, from, fromlen, to, tolen)
 	size_t buflen;
 	int flags;
 	struct sockaddr *from;
-	u_int *fromlen;
+	socklen_t *fromlen;
 	struct sockaddr *to;
 	u_int *tolen;
 {
